{% load i18n sitetree %}
<style type="text/css">
    .ctl { width: 14px; height: 12px; display: inline-block;}
    .arr_up { background: url('{{ STATIC_URL }}admin/img/sorting-icons.gif') -5px -50px no-repeat; }
    .arr_dn { background: url('{{ STATIC_URL }}admin/img/sorting-icons.gif') -5px -94px no-repeat; }
</style>
{% for item in sitetree_items %}
	<tr>
		<td class="align-center"><img src="{{ STATIC_URL }}admin/img/icon-{{ item.hidden|yesno:"yes,no" }}.gif" alt="{{ item.hidden }}" /></td>
		<td class="align-center"><img src="{{ STATIC_URL }}admin/img/icon-{{ item.inmenu|yesno:"yes,no" }}.gif" alt="{{ item.inmenu }}" /></td>
		<td class="align-center"><img src="{{ STATIC_URL }}admin/img/icon-{{ item.inbreadcrumbs|yesno:"yes,no" }}.gif" alt="{{ item.inbreadcrumbs }}" /></td>
		<td class="align-center"><img src="{{ STATIC_URL }}admin/img/icon-{{ item.insitetree|yesno:"yes,no" }}.gif" alt="{{ item.insitetree }}" /></td>
		<td class="align-center"><img src="{{ STATIC_URL }}admin/img/icon-{{ item.access_restricted|yesno:"yes,no" }}.gif" alt="{{ item.access_restricted }}" /></td>
        <td class="align-center"><img src="{{ STATIC_URL }}admin/img/icon-{{ item.access_loggedin|yesno:"yes,no" }}.gif" alt="{{ item.access_loggedin }}" /></td>
		<td>
		{% for d in item.depth_range %}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{% endfor %}
		{% if item.parent %}|&#8212;{% endif %}
		<a href="item_{{ item.id }}/">{{ item.title }}</a></td>
		<td>{{ item.url }}</td>
		<td>
			&nbsp;&nbsp;
			<a href="item_{{ item.id }}/move_up/"><img src="{{ STATIC_URL }}admin/img/arrow-up.gif" class="ctl arr_up" border="0" title="{% trans "Move up" %}"/></a>
			&nbsp;&nbsp;&nbsp;
			<a href="item_{{ item.id }}/move_down/"><img src="{{ STATIC_URL }}admin/img/arrow-down.gif" class="ctl arr_dn" border="0" title="{% trans "Move down" %}"/></a>
		</td>
	</tr>
	{% if item.has_children %}
		{% sitetree_children of item for sitetree template "admin/sitetree/tree/tree.html" %}
	{% endif %}
{% endfor %}
