{% load sitetree %}
<ul class="nav-bar vertical">
	{% for item in sitetree_items %}
	<li class="{% if item.is_current or item.in_current_branch %}active{% endif %} {% if item.has_children %}has-flyout{% endif %}">
        <a href="{% if item.has_children %}#{% else %}{% sitetree_url for item %}{% endif %}">{{ item.title_resolved }}</a>
		{% if item.has_children %}
            <a href="#" class="flyout-toggle"><span> </span></a>
			{% sitetree_children of item for menu template "sitetree/menu_foundation_flyout.html" %}
		{% endif %}
	</li>
	{% endfor %}
</ul>